/***************************************************************************************************
* Program:    Super Nintendo Entertainment System(tm) Audio Processing Unit Emulator DLL           *
* Platform:   Intel 80386 & MMX                                                                    *
* Programmer: Anti Resonance                                                                       *
*                                                                                                  *
* "SNES" and "Super Nintendo Entertainment System" are trademarks of Nintendo Co., Limited and its *
* subsidiary companies.                                                                            *
*                                                                                                  *
* This program is free software; you can redistribute it and/or modify it under the terms of the   *
* GNU General Public License as published by the Free Software Foundation; either version 2 of     *
* the License, or (at your option) any later version.                                              *
*                                                                                                  *
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;        *
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.        *
* See the GNU General Public License for more details.                                             *
*                                                                                                  *
* You should have received a copy of the GNU General Public License along with this program;       *
* if not, write to the Free Software Foundation, Inc.                                              *
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                                        *
*                                                                                                  *
* ------------------------------------------------------------------------------------------------ *
* Revision History:                                                                                *
*                                                                                                  *
* 2.0  26.01.2004                                                                                  *
*      + Added BitRateReduce, a better function for creating BBR blocks from raw samples           *
*                                                                                                  *
* 0.98 10.02.2003 SNESAmp v2.99                                                                    *
*      + Added version information to the DLL                                                      *
*      + Moved GetAPUData to SNESAPU.cpp, since it's only needed for SNESAPU.DLL                   *
*      + Moved memory allocation into SNESAPU.cpp, since memory allocation is system, and possibly *
*        application, specific                                                                     *
*                                                                                                  *
* 0.96                                                                                             *
*                                                                                                  *
* 0.95 15.05.2001                                                                                  *
*                                                                                                  *
* 0.92 03.03.2001 SNESAmp v2.1                                                                     *
*                                                                                                  *
* 0.90 01.01.2001 SNESAmp v2.0                                                                     *
*      + Made some changes to the internal interface with the DSP                                  *
*                                                                                                  *
* 0.85 14.12.2000 Super Jukebox v3.0                                                               *
*                                                      Copyright (C)2001-2004 Alpha-II Productions *
***************************************************************************************************/

#ifndef	__INC_SNESAPU
#define	__INC_SNESAPU

#define	SNESAPU_DLL								//Don't include local function/variable definitions

#include	"APU.h"
#include	"SPC700.h"
#include	"DSP.h"

//**************************************************************************************************
//Function pointers to SNESAPU

typedef struct
{
	u8		*ram;								//Base pointer to APU RAM
	u8		*xram;								//Pointer to writeable IPL region
	u8		*outPort;							//SPC700 output ports
	u32		*t64Cnt;							//64kHz timer counter
	DSPReg	*dsp;								//DSP registers
	Voice	*voice;								//Internal DSP mixing data
	u32		*vMMaxL,*vMMaxR;					//Max main sample output

	void	(__stdcall *SNESAPUInfo)(u32 *pVer, u32 *pMin, u32 *pOpt);

	void	(__stdcall *GetAPUData)(u8 **pRAM, u8 **pXRAM, u8 **pSPCOut, u32 **ppT64Cnt, \
									DSPReg **ppDSP, Voice **pVoice, u32 **ppVMMaxL, u32 **ppVMMaxR);
	void	(__stdcall *BitRateReduce)(void *pBlk, void *pSmp, u32 *pLen, u32 *pLoop, s32 opt);
	void	(__stdcall *ResetAPU)(u32 amp);
	void	(__stdcall *FixAPU)(u16 pc, u8 a, u8 y, u8 x, u8 psw, u8 sp);
	void	(__stdcall *LoadSPCFile)(void *pSPC);
	void	(__stdcall *SaveAPU)(SPCState *pSPC, DSPState *pDSP);
	void	(__stdcall *RestoreAPU)(SPCState *pSPC, DSPState *pDSP);
	void	(__stdcall *SetAPUOpt)(u32 mix, u32 chn, u32 bits, u32 rate, u32 inter, u32 opts);
	void	(__stdcall *SetAPUSmpClk)(u32 speed);
	u32		(__stdcall *SetAPULength)(u32 song, u32 fade);
	void	(__stdcall *SetAPURAM)(u32 addr, u8 val);
	void*	(__stdcall *EmuAPU)(void *pBuf, u32 len, b8 type);
	void	(__stdcall *SeekAPU)(u32 time, b8 fast);
	SPCDebug*	(__stdcall *SetSPCDbg)(SPCDebug *pTrace, u32 opts);
	void	(__stdcall *SetTimerTrick)(u8 port, u32 time);
	void	(__stdcall *GetSPCRegs)(u16 *pPC, u8 *pA, u8 *pY, u8 *pX, u8 *pPSW, u8 *pSP);
	void	(__stdcall *InPort)(u32 port, u32 val);
	void	(__stdcall *InPortW)(u32 port, u32 val);
	s32		(__stdcall *EmuSPC)(u32 cyc);
	u32		(__stdcall *GetProcInfo)();
	void	(__stdcall *GetScript700Data)(u32 *pVer, u8 **ppProgram, u32 **ppWork, u16 **ppPointer, u32 **ppFlag1, u32 **ppFlag2, u32 **ppWaitCnt, u8 **ppDSPFlag, u8 **ppChange, u32 **ppDetune);
	u32		(__stdcall *SetScript700)(void *pSource);
	DSPDebug*	(__stdcall *SetDSPDbg)(DSPDebug *pTrace);
	void	(__stdcall *SetDSPReg)(u8 reg, u8 val);
	void	(__stdcall *SetDSPPitch)(u32 base);
	void	(__stdcall *SetDSPAmp)(u32 level);
	void	(__stdcall *SetDSPStereo)(u32 sep);
	void	(__stdcall *SetDSPEFBCT)(s32 leak);
	void*	(__stdcall *EmuDSP)(void *pBuf, s32 size);
	void*	(__stdcall *PlayWave)(void *pSrc, u32 loop, u32 rate);
	u32		(__stdcall *UnpackWave)(s16 *pBuf, void *pSrc, u32 num, u32 opts, s32 *prev1, s32 *prev2);
	void	(__stdcall *PackWave)(void *pBuf, s16 *pSmp, u32 num, u32 opts, s32 *prev1, s32 *prev2);
	void	(__stdcall *VMax2dB)(void *pList, b8 fp);

	s32 	(__stdcall *try700)(char *fname);
	void 	(__stdcall *InitWork_700)();

} SAPUFunc;


//**************************************************************************************************
//External Functions - See other header files for exported function descriptions

#define	import	__declspec(dllimport)

#ifdef	__cplusplus
extern "C" {
#endif

//**************************************************************************************************
// SNESAPU Information
//
// Desc:
//    Returns the version and compile information about the library
//
// In:
//    pVer -> Version
//            bits 7-0   Optional letter (a-z, 0x00 otherwise)
//            bits 15-8  Minor version (stored as BCD)
//            bits 23-16 Major version (stored as BCD)
//            bits 31-24 0x00
//            ex. 0x00015061 = 1.50a
//    pMin -> Minimum version, the oldest version this library is backwards compatible with
//    pOpt -> Compile options
//            bits 23-0  Directives (see defines at top of APU.h)
//            bits 31-24 SPC700 clock divisor

import	void	__stdcall SNESAPUInfo(u32 *pVer, u32 *pMin, u32 *pOpt);


//**************************************************************************************************
// Get Pointers to APU Data
//
// Desc:
//    Returns pointers to public data
//
// In:
//    NULL = Return nothing
//
// Out:
//    pRAM       -> 64K of RAM
//    pXRAM      -> 64-bytes of RAM masked by IPL ROM region
//    pOutPort   -> 4-bytes used by the SPC700 to write back to the 65816
//    ppT64Cnt   -> 32-bit counter increased every 64kHz
//    ppDSP      -> 128-bytes of DSP registers
//    pVoice     -> Array of 8 Voice structures used by the DSP emulator
//    ppVMMaxL/R -> 32-bit absolute peak output before clipping (this value is relative to 16-bit
//                   values, even when 32-bit mixing is selected)


import	void	__stdcall GetAPUData(u8 **pRAM, u8 **pXRAM, u8 **pOutPort, u32 **ppT64Cnt, \
									 DSPReg **ppDSP, Voice **pVoice, u32 **ppVMMaxL, u32 **ppVMMaxR);


//**************************************************************************************************
// Compress Samples Using SNES BRR
//
// In:
//    pBlk -> Buffer to store BBR blocks
//    pSmp -> Samples to compress (can be the same as pBuf)
//    pLen -> Length of sound (in samples)
//    pLoop-> Loop point (in samples, -1 if no loop)
//    opt   = Options (see BRR_??? #defines in DSP.h)
//
// Out:
//    pLen  = Length of source in BRR blocks
//    pLoop = Index of loop start block (-1 if no loop)

import	void	__stdcall BitRateReduce(void *pBlk, void *pSmp, u32 *pLen, u32 *pLoop, s32 opt);


import	void	__stdcall ResetAPU(u32 amp);
import	void	__stdcall FixAPU(u16 pc, u8 a, u8 y, u8 x, u8 psw, u8 sp);
import	void	__stdcall LoadSPCFile(void *pSPC);
import	void	__stdcall SaveAPU(SPCState *pSPC, DSPState *pDSP);
import	void	__stdcall RestoreAPU(SPCState *pSPC, DSPState *pDSP);
import	void	__stdcall SetAPUOpt(u32 mix, u32 chn, u32 bits, u32 rate, u32 inter, u32 opts);
import	void	__stdcall SetAPUSmpClk(u32 speed);
import	u32		__stdcall SetAPULength(u32 song, u32 fade);
import	void	__stdcall SetAPURAM(u32 addr, u8 val);
import	void*	__stdcall EmuAPU(void *pBuf, u32 len, b8 type);
import	void	__stdcall SeekAPU(u32 time, b8 fast);
import	SPCDebug*	__stdcall SetSPCDbg(SPCDebug *pTrace, u32 opts);
import	void	__stdcall SetTimerTrick(u8 port, u32 time);
import	void	__stdcall GetSPCRegs(u16 *pPC, u8 *pA, u8 *pY, u8 *pX, u8 *pPSW, u8 *pSP);
import	void	__stdcall InPort(u32 port, u32 val);
import	void	__stdcall InPortW(u32 port, u32 val);
import	s32		__stdcall EmuSPC(u32 cyc);
import	u32		__stdcall GetProcInfo();
import	void	__stdcall GetScript700Data(u32 *pVer, u8 **ppProgram, u32 **ppWork, u16 **ppPointer, u32 **ppFlag1, u32 **ppFlag2, u32 **ppWaitCnt, u8 **ppDSPFlag, u8 **ppChange, u32 **ppDetune);
import	u32		__stdcall SetScript700(void *pSource);
import	DSPDebug*	__stdcall SetDSPDbg(DSPDebug *pTrace);
import	void	__stdcall SetDSPReg(u8 reg, u8 val);
import	void	__stdcall SetDSPPitch(u32 base);
import	void	__stdcall SetDSPAmp(u32 level);
import	void	__stdcall SetDSPStereo(u32 sep);
import	void	__stdcall SetDSPEFBCT(s32 leak);
import	void*	__stdcall EmuDSP(void *pBuf, s32 size);
import	void*	__stdcall PlayWave(void *pSrc, u32 loop, u32 rate);
import	u32		__stdcall UnpackWave(s16 *pBuf, u32 ofs, u32 num, u32 opts, s32 *prev1, s32 *prev2);
import	void	__stdcall PackWave(void *pBuf, s16 *pSmp, u32 num, u32 opts, s32 *prev1, s32 *prev2);
import	void	__stdcall VMax2dB(void *pList, b8 fp);

import	s32 	__stdcall try700(char *fname);
import	void 	__stdcall InitWork_700();

#ifdef	__cplusplus
}
#endif

#endif	//__INC_SNESAPU
